@@ -4,16 +4,29 @@
 #    combination with a non-existant file name. See the libcurl docs
 #    for more info.
 #
-# If you want thread-safe operation, you'll have to set the NOSIGNAL option
-# yourself.
-#
 # By Eric S. Raymond, April 2003.
 
-import os, sys, urllib, exceptions, mimetools, pycurl
+import sys, exceptions, mimetools, pycurl
+try:
+    import urllib.parse as urllib_parse
+    from urllib.parse import urljoin
+except ImportError:
+    import urllib as urllib_parse
+    from urlparse import urljoin
 try:
     from cStringIO import StringIO
 except ImportError:
-    from StringIO import StringIO
+    try:
+        from StringIO import StringIO
+    except ImportError:
+        from io import StringIO
+
+try:
+    import signal
+    from signal import SIGPIPE, SIG_IGN
+    signal.signal(signal.SIGPIPE, signal.SIG_IGN)
+except ImportError:
+    pass
 
 
 class Curl:
@@ -32,6 +45,7 @@ class Curl:
         # Follow redirects in case it wants to take us to a CGI...
         self.set_option(pycurl.FOLLOWLOCATION, 1)
         self.set_option(pycurl.MAXREDIRS, 5)
+        self.set_option(pycurl.NOSIGNAL, 1)
         # Setting this option with even a nonexistent file makes libcurl
         # handle cookie capture and playback automatically.
         self.set_option(pycurl.COOKIEFILE, "/dev/null")
@@ -58,7 +72,7 @@ class Curl:
 
     def set_option(self, *args):
         "Set an option on the retrieval."
-        apply(self.handle.setopt, args)
+        self.handle.setopt(*args)
 
     def set_verbosity(self, level):
         "Set verbosity to 1 to see transactions."
@@ -69,7 +83,7 @@ class Curl:
         if self.fakeheaders:
             self.set_option(pycurl.HTTPHEADER, self.fakeheaders)
         if relative_url:
-            self.set_option(pycurl.URL,os.path.join(self.base_url,relative_url))
+            self.set_option(pycurl.URL, urljoin(self.base_url, relative_url))
         self.payload = ""
         self.hdr = ""
         self.handle.perform()
@@ -78,14 +92,14 @@ class Curl:
     def get(self, url="", params=None):
         "Ship a GET request for a specified URL, capture the response."
         if params:
-            url += "?" + urllib.urlencode(params)
+            url += "?" + urllib_parse.urlencode(params)
         self.set_option(pycurl.HTTPGET, 1)
         return self.__request(url)
 
     def post(self, cgi, params):
         "Ship a POST request to a specified CGI, capture the response."
         self.set_option(pycurl.POST, 1)
-        self.set_option(pycurl.POSTFIELDS, urllib.urlencode(params))
+        self.set_option(pycurl.POSTFIELDS, urllib_parse.urlencode(params))
         return self.__request(cgi)
 
     def body(self):
@@ -98,7 +112,7 @@ class Curl:
 
     def get_info(self, *args):
         "Get information about retrieval."
-        return apply(self.handle.getinfo, args)
+        return self.handle.getinfo(*args)
 
     def info(self):
         "Return a dictionary with all info on the last response."
@@ -133,6 +147,8 @@ class Curl:
         m['num-connects'] = self.handle.getinfo(pycurl.NUM_CONNECTS)
         m['ssl-engines'] = self.handle.getinfo(pycurl.SSL_ENGINES)
         m['cookielist'] = self.handle.getinfo(pycurl.INFO_COOKIELIST)
+        m['lastsocket'] = self.handle.getinfo(pycurl.LASTSOCKET)
+        m['ftp-entry-path'] = self.handle.getinfo(pycurl.FTP_ENTRY_PATH)
         return m
 
     def answered(self, check):
@@ -157,10 +173,10 @@ if __name__ == "__main__":
         url = sys.argv[1]
     c = Curl()
     c.get(url)
-    print c.body()
-    print '='*74 + '\n'
+    print(c.body())
+    print('='*74 + '\n')
     import pprint
     pprint.pprint(c.info())
-    print c.get_info(pycurl.OS_ERRNO)
-    print c.info()['os-errno']
+    print(c.get_info(pycurl.OS_ERRNO))
+    print(c.info()['os-errno'])
     c.close()
